"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Shard = exports.ShardKind = void 0;
const tslib_1 = require("tslib");
const cid_1 = require("multiformats/cid");
const buffer_1 = require("buffer");
const errors_1 = require("./errors");
const events_1 = require("events");
const _1 = require(".");
const it_all_1 = (0, tslib_1.__importDefault)(require("it-all"));
var ShardKind;
(function (ShardKind) {
    // eslint-disable-next-line no-unused-vars
    ShardKind["Binary"] = "binary";
    // eslint-disable-next-line no-unused-vars
    ShardKind["Object"] = "object";
})(ShardKind = exports.ShardKind || (exports.ShardKind = {}));
class Shard {
    constructor(stack, cid, kind, data) {
        this.stack = stack;
        this.cid = cid;
        this.kind = kind;
        this.data = data;
        this.events = new events_1.EventEmitter();
    }
    put(data) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            switch (this.kind) {
                case ShardKind.Binary:
                    this.cid = (yield this.stack.ipfs.add(data)).cid;
                    break;
                case ShardKind.Object:
                    this.cid = yield this.stack.ipfs.dag.put(data);
                    break;
                default:
                    throw new errors_1.UnknownShardKindError();
            }
            this.events.emit('update', this);
        });
    }
    on(event, listener) {
        this.events.on(event, listener);
    }
    static new(stack, kind, data) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const cid = yield stack.ipfs.dag.put({ empty: true });
            const shard = new Shard(stack, cid, kind, data);
            shard.put(data);
            return shard;
        });
    }
    static create(stack, cid, kind) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            let data;
            switch (kind) {
                case ShardKind.Binary:
                    data = buffer_1.Buffer.from(yield (0, it_all_1.default)(stack.ipfs.cat(cid)));
                    break;
                case ShardKind.Object:
                    data = (yield stack.ipfs.dag.get(cid)).value;
                    break;
                default:
                    throw new errors_1.UnknownShardKindError();
            }
            return new Shard(stack, cid, kind, data);
        });
    }
    static from(stack, path) {
        // eslint-disable-next-line no-unused-vars
        const [_, shard, cid, kind] = path.split('/');
        if (shard !== 'shard') {
            throw new _1.InvalidShardPathError();
        }
        try {
            return Shard.create(stack, cid_1.CID.parse(cid), kind);
        }
        catch (error) {
            throw new _1.InvalidShardPathError(error);
        }
    }
    toString() {
        return `/shard/${this.cid.toString()}/${this.kind}`;
    }
}
exports.Shard = Shard;
//# sourceMappingURL=shard.js.map